#******************************************************************************
#
#       Copyright:      2005-2006 Paul Obermeier (obermeier@poSoft.de)
#
#                       See the file "Tcl3D_License.txt" for information on
#                       usage and redistribution of this file, and for a
#                       DISCLAIMER OF ALL WARRANTIES.
#
#       Module:         Tcl3D -> tcl3dUtil
#       Filename:       tcl3dWidgets.tcl
#
#       Author:         Paul Obermeier
#
#       Description:    Tcl module implementing some simple Tk widgets like
#                       scrolled listboxes and text widgets, as well as
#                       some widget and window handling utilities.
#
#******************************************************************************

# Determine, if the windowing system is Apple's Aqua.
proc tcl3dHaveAqua {} {
    if { [tk windowingsystem] eq "aqua" } {
        return 1
    } else {
        return 0
    }
}

# Determine, if we want to show the indicators for radio- and checkbuttons.
# Currently we do this on a Mac running Aqua, because it looks very 
# buggy otherwise.
proc tcl3dShowIndicator {} {
    if { [tcl3dHaveAqua] } {
        return 1
    } else {
        return 0
    }
}

proc tcl3dWinIsTop { path } {
    string equal $path [winfo toplevel $path]
}

proc tcl3dWinRaise { tw } {
    wm deiconify $tw
    update
    raise $tw
}

proc tcl3dSetFullScreenMode { win } {
    set sh [winfo screenheight $win]
    set sw [winfo screenwidth  $win]

    wm minsize $win $sw $sh
    wm maxsize $win $sw $sh
    set fmtStr [format "%dx%d+0+0" $sw $sh]
    wm geometry $win $fmtStr
    wm overrideredirect $win 1
    focus -force $win
}

proc tcl3dSetWindowMode { win w h } {
    set sh [winfo screenheight $win]
    set sw [winfo screenwidth  $win]

    wm minsize $win 10 10
    wm maxsize $win $sw $sh
    set fmtStr [format "%dx%d+0+25" $w $h]
    wm geometry $win $fmtStr
    wm overrideredirect $win 0
    focus -force $win
}

proc tcl3dSetScrolledTitle { w titleStr { fgColor "black" } } {
    set pathList [split $w "."]
    # Index -3 is needed for CreateScrolledFrame.
    # Index -2 is needed for all other widget types.
    foreach ind { -2 -3 } {
        set parList  [lrange $pathList 0 [expr [llength $pathList] $ind]]
        set parPath  [join $parList "."]

        set labelPath $parPath
        append labelPath ".label"
        if { [winfo exists $labelPath] } {
            $labelPath configure -text $titleStr -foreground $fgColor
            break
        }
    }
}

proc tcl3dCreateScrolledWidget { wType w titleStr args } {
    if { [winfo exists $w.par] } {
        destroy $w.par
    }
    frame $w.par
    pack $w.par -side top -fill both -expand 1
    if { [string compare $titleStr ""] != 0 } {
        label $w.par.label -text "$titleStr"
    }
    eval { $wType $w.par.widget \
            -xscrollcommand "$w.par.xscroll set" \
            -yscrollcommand "$w.par.yscroll set" } $args
    scrollbar $w.par.xscroll -command "$w.par.widget xview" -orient horizontal
    scrollbar $w.par.yscroll -command "$w.par.widget yview" -orient vertical
    set rowNo 0
    if { [string compare $titleStr ""] != 0 } {
        set rowNo 1
        grid $w.par.label -sticky ew -columnspan 2
    }
    grid $w.par.widget $w.par.yscroll -sticky news
    grid $w.par.xscroll               -sticky ew

    grid rowconfigure    $w.par $rowNo -weight 1
    grid columnconfigure $w.par 0      -weight 1

    ::tcl3dAutoscroll::autoscroll $w.par.xscroll
    ::tcl3dAutoscroll::autoscroll $w.par.yscroll

    return $w.par.widget
}

proc tcl3dScrolledFrameCfgCB {w width height} {
    set newSR [list 0 0 $width $height]
    if { ! [string equal [$w.canv cget -scrollregion] $newSR] } {
        $w.canv configure -scrollregion $newSR
    }
}

proc tcl3dCreateScrolledFrame { w titleStr args } {
    frame $w.par
    pack $w.par -fill both -expand 1
    if { [string compare $titleStr ""] != 0 } {
        label $w.par.label -text "$titleStr" -borderwidth 2 -relief raised
    }
    eval {canvas $w.par.canv -xscrollcommand [list $w.par.xscroll set] \
                             -yscrollcommand [list $w.par.yscroll set]} $args
    scrollbar $w.par.xscroll -orient horizontal -command "$w.par.canv xview"
    scrollbar $w.par.yscroll -orient vertical   -command "$w.par.canv yview"
    set fr [frame $w.par.canv.fr -borderwidth 0 -highlightthickness 0]
    $w.par.canv create window 0 0 -anchor nw -window $fr

    set rowNo 0
    if { [string compare $titleStr ""] != 0 } {
        set rowNo 1
        grid $w.par.label -sticky ew -columnspan 2
    }
    grid $w.par.canv $w.par.yscroll -sticky news
    grid $w.par.xscroll             -sticky ew
    grid rowconfigure    $w.par $rowNo -weight 1
    grid columnconfigure $w.par 0      -weight 1
    # This binding makes the scroll-region of the canvas behave correctly as
    # you place more things in the content frame.
    bind $fr <Configure> [list tcl3dScrolledFrameCfgCB $w.par %w %h]
    $w.par.canv configure -borderwidth 0 -highlightthickness 0
    return $fr
}

proc tcl3dCreateScrolledListbox { w titleStr args } {
    return [eval {tcl3dCreateScrolledWidget listbox $w $titleStr} $args ]
}

proc tcl3dCreateScrolledText { w titleStr args } {
    return [eval {tcl3dCreateScrolledWidget text $w $titleStr} $args ]
}

proc tcl3dCreateScrolledCanvas { w titleStr args } {
    return [eval {tcl3dCreateScrolledWidget canvas $w $titleStr} $args ]
}

proc tcl3dCreateScrolledTable { w titleStr args } {
    return [eval {tcl3dCreateScrolledWidget table $w $titleStr} $args ]
}

proc tcl3dCreateScrolledTablelist { w titleStr args } {
    return [eval {tcl3dCreateScrolledWidget tablelist::tablelist $w $titleStr} $args ]
}
